searchData={"items":[{"type":"module","title":"TypedStructCtor","doc":"# TypedStructCtor\n\nA TypedStruct plugin to add validating constructors to a TypedStruct module\n\nThe [TypedStruct](https://hexdocs.pm/typedstruct/TypedStruct.html) macro wraps field definitions to reduce boilerplate\nneeded to define elixir structs and provides a\n[plugin system](https://hexdocs.pm/typedstruct/TypedStruct.Plugin.html) for clients to extend the DSL.\n\n`TypedStructCtor` uses the `__changeset__` \"reflection\" function added by the plugin\n[TypedStructEctoChangeset](https://hexdocs.pm/typed_struct_ecto_changeset/TypedStructEctoChangeset.html) which enables\nEcto.Changeset.cast on fields defined within the TypedStruct macro.","ref":"TypedStructCtor.html"},{"type":"module","title":"Try it out in Livebook - TypedStructCtor","doc":"Try the macro out in real time without having to install or write any of your own code\n\nTo get started you need a running instance of [Livebook](https://livebook.dev/)\n\n[![Run in Livebook](https://livebook.dev/badge/v1/blue.svg)](https://livebook.dev/run?url=https://github.com/withbelay/typed_struct_ctor/blob/main/try_it_out.livemd)","ref":"TypedStructCtor.html#module-try-it-out-in-livebook"},{"type":"module","title":"Rationale - TypedStructCtor","doc":"[Ecto.Changeset](https://hexdocs.pm/ecto/Ecto.Changeset.html) is a great way to create validated\nstructs.  However, if you've many validated structs they quickly become \"noise\", where writing\nthese functions can be tedious, and bugs are easily introduced as struct changes are easily overlooked.\nWhen the effort needed to write boilerplate tests for boilerplate code is factored in, it can be tempting to skip\nstruct validation altogether.","ref":"TypedStructCtor.html#module-rationale"},{"type":"module","title":"Simple Examples - TypedStructCtor","doc":"```elixir\n    defmodule AStruct do\n      use TypedStruct\n\n      typedstruct do\n        plugin(TypedStructEctoChangeset)\n        plugin(TypedStructCtor)\n\n        # A required field with a default value provided by MFA tuple to return a UUID\n        field :id, :string, default_apply: {Ecto.UUID, :generate, []}\n        \n        # A required field with no default, meaning it must be provided to the constructor.\n        # It's an `integer` with known `Ecto.cast` behavior, so for instance, string values are cast \n        # for example string to integer\n        field :integer_field, :integer\n        \n        # An optional field with no default, meaning it will only have a value if provided to the \n        # constructor\n        field :some_string, :string, required: false \n      end\n    end\n\n    iex()> AStruct.new(%{some_string: \"foo\"})\n    {:error,\n     #Ecto.Changeset ,\n       valid?: false\n     >}\n\n    # With `bang` notation and demonstrating Ecto's field cast (string to integer)\n    iex()> AStruct.new!(%{some_string: \"bar\", integer_field: \"42\"})\n    %AStruct{\n      some_string: \"bar\",\n      integer_field: 42,\n      id: \"2e28df41-c024-465e-901d-22c974f1d356\"\n    }\n  ```\n\nThe TypedStruct macro makes it much easier to define structs.  The TypedStructEctoChangeset plugin uses the field\ndefinitions to generate an Ecto.Changeset.cast function for fields in the struct.  And this plugin, TypedStructCtor,\nuses those `cast` functions to generate validating constructors for the enclosing struct created by TypedStruct.\n\nThis plugin adds 5 constructors, `new/0`, `new/1`, `new!/1`, `from/2`, and `from!/2` to the given module.  \nEcto `cast` is called for all attributes provided to the constructors, defaults are applied where needed, and\nvalidation is performed.  \n\nThe `new` functions return {:ok, struct} or {:error, changeset}, while the `new!` functions return the struct, or\nraises if there were issues with `cast` or validation.\n\nThe `new` function takes an optional map of attributes, does\n[Changeset.cast](https://hexdocs.pm/ecto/Ecto.Changeset.html#cast/4) of all values matching the defined\nfields, adds defaults for fields missing values, validates any required fields, and finally does\n[Changeset.apply_action](https://hexdocs.pm/ecto/Ecto.Changeset.html#apply_action/2) to validate the changeset.\nReturns `{:ok,  }` if everything is OK, `{:error,  }` if there were issues with `cast` or validation.\nBecause it's necessary to properly handle mappable fields, if a struct is passed to the `new` function,\n`{:error, :attributes_must_be_a_map}` is returned; use one of the `from` functions for that use case as described below.\n\nThe `from` functions are useful in messaging environments where a new message is created from a some set of\nvalues from a source message.  They are similar to the `new` functions but accept a \"base struct\" as the first argument \nand a map of attributes as the second argument.  The base struct is mapped first to the field values, and the attributes \nare merged on top.","ref":"TypedStructCtor.html#module-simple-examples"},{"type":"module","title":"Required Fields - TypedStructCtor","doc":"By default, all fields are required when calling the constructors.  Meaning you'll get a changeset error if the \nfield does not have a default, and you don't provide an attribute value for it in the constructor.\n\nYou can override this by passing `required: false` to the plugin\n\n  ```elixir\n  typedstruct do\n    plugin(TypedStructEctoChangeset)\n    plugin(TypedStructCtor, required: false)\n\n    field :this_field_is_not_required, :string\n  end\n  ```\n\nOr by passing `required: false` to the `field` definition.\n\n  ```elixir\n  typedstruct do\n    plugin(TypedStructEctoChangeset)\n    plugin(TypedStructCtor)\n\n    field :this_field_is_not_required, :string, required: false\n  end\n  ```","ref":"TypedStructCtor.html#module-required-fields"},{"type":"module","title":"Field-level Defaults - TypedStructCtor","doc":"`default` and `default_apply` can be provided to the `field` definition to specify a default value for the field.\n\nThough you can specify both `default` and `default_apply` (an MFA tuple), only one will be used.\n`default` will be used with Elixir's struct syntax (e.g. `%AStruct{}`).\n`default_apply` will be invoked when one of the 5 constructor functions is used (e.g. `AStruct.new!()`)\n\nThe `default_apply` function is short-circuited and will only be invoked if the given field was not present in the \nattributes.\n\n  ```elixir\n    defmodule AStructWithDefaulting do\n      use TypedStruct\n\n      typedstruct do\n        plugin(TypedStructEctoChangeset)\n        plugin(TypedStructCtor)\n\n        field :field, :integer, default: 42, default_apply: {SomeModule, :some_function, [\"55\"]}\n      end\n    end\n\n    iex()> %AStructWithDefaulting{}\n    %AStructWithDefaulting{field: 42}\n\n    iex()> AStructWithDefaulting.new!()\n    %AStructWithDefaulting{field: 55}\n  ```","ref":"TypedStructCtor.html#module-field-level-defaults"},{"type":"module","title":"Mappable Fields - TypedStructCtor","doc":"As mentioned above, when using `from` and `from!` functions, the first argument is a \"source\" struct whose matching-name\nfields will be copied first into the struct being constructed.  By default, all matching-name fields are copied, but\nthe `mappable?` boolean attribute can be used to specify which fields are not copied.  This is useful when you want the\nnewly constructed struct to have different values for a field than the source struct such as `created_at` or `id`.\n\nNot mapping over the source struct values will mean the newly constructed struct will leave the new fields empty\nunless defaulted or provided as attributes to the constructor.\n\n  ```elixir\n    defmodule AStructWithMappableFields do\n      use TypedStruct\n    \n      typedstruct do\n        plugin(TypedStructEctoChangeset)\n        plugin(TypedStructCtor)\n    \n        field(:id, :string, mappable?: false, default_apply: {Ecto.UUID, :generate, []})\n        field(:created_at, :utc_datetime_usec, mappable?: false, default_apply: {DateTime, :utc_now, []})\n        field(:reason, :string)\n      end\n    end\n\n    # In the example below, the `id` and `created_at` fields are `mappable?: false` so they are not \n    # copied from the source struct.  So in the new struct, `:reason` is copied from the source \n    # struct, `:id` is provided in the attributes map, and `:created_at`, being nil after all the\n    # copying is done, causes its default to be used instead, resulting in a new date.\n    iex()> source_struct = AStructWithMappableFields.new!(%{reason: \"because\"})\n    %AStructWithMappableFields{\n      reason: \"because\",\n      created_at: ~U[2023-11-18 04:57:16.754681Z],\n      id: \"ffe94776-5d6e-4d84-9aeb-2862d874577f\"\n    }\n    \n    iex()>  Process.sleep(5)\n    iex()>  mapped = AStructWithMappableFields.from!(source_struct, %{id: \"id from attributes\"})\n    %AStructWithMappableFields{\n      reason: \"because\",\n      created_at: ~U[2023-11-18 04:57:16.766353Z],\n      id: \"id from attributes\"\n    }\n\n    iex()> Process.sleep(5)\n    iex()> mapped = AStructWithMappableFields.from!(source_struct, %{reason: \"I said so\"})\n    %AStructWithMappableFields{\n      reason: \"I said so\",\n      created_at: ~U[2023-11-18 04:57:16.772312Z],\n      id: \"a09be86b-373a-48f0-9d74-faee10037421\"\n    }\n  ```","ref":"TypedStructCtor.html#module-mappable-fields"},{"type":"module","title":"Installation - TypedStructCtor","doc":"Because this plugin supports the interface defined by the `TypedStruct` macro, installation assumes you've already\nadded that dependency.\n\nWhile you can use the original [typed_struct](https://hex.pm/packages/typed_struct) library, it seems to no longer be\nmaintained.  However, there is a fork [here](https://hex.pm/packages/typedstruct) that is quite active.\n\nThe package can be installed by adding `typed_struct_ctor` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    # Choose either of the following `TypedStruct` libraries \n    # both use the same name for the macro - `typedstruct` but\n    # but are mutually exclusive:\n    \n    # The original, but no longer maintained library\n    {:typed_struct, \"~> 0.3.0\"},\n      \n    # Or the newer forked library\n    {:typedstruct, \"~> 0.5.2\"},\n\n    # And add this library  \n    {:typed_struct_ctor, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"TypedStructCtor.html#module-installation"},{"type":"function","title":"TypedStructCtor.from/1","doc":"Create a new struct where values from fields in the provided struct are copied.\n\nThe `from/1` constructor is useful for event driven systems where it is common to create\na new event from a given \"triggering\" event\n\n* Provided values from `base_struct` will be cast to the appropriate field type.\n* Fields in the newly constructed struct that are not provided map will\n  be set to their default values.\n* Required fields that are nil will result in a Ecto.Changeset error","ref":"TypedStructCtor.html#from/1"},{"type":"function","title":"TypedStructCtor.from/2","doc":"Create a new struct where values from fields in the provided struct are copied, then values\nfrom the provided attributes map are copied to like-named fields in the struct; overwriting\nany values copied from the base_struct.\n\nThe `from/2` constructor is useful for event driven systems where it is common to create\na new event from a given \"triggering\" event\n\n* Provided values from `base_struct` will be cast to the appropriate field type.\n* Fields in the newly constructed struct that are not provided map will\n  be set to their default values.\n* Required fields that are nil will result in a Ecto.Changeset error","ref":"TypedStructCtor.html#from/2"},{"type":"function","title":"TypedStructCtor.from!/1","doc":"Create a new struct where values from fields in the provided struct are copied.  Raising if the new struct\ncannot be validated.\n\nThe `from!/1` constructor is useful for event driven systems where it is common to create\na new event from a given \"triggering\" event\n\n* Provided values from `base_struct` will be cast to the appropriate field type.\n* Fields in the newly constructed struct that are not provided map will\n  be set to their default values.\n* Any cast error or missing required field will result in a `raise`","ref":"TypedStructCtor.html#from!/1"},{"type":"function","title":"TypedStructCtor.from!/2","doc":"Create a new struct where values from fields in the provided struct are copied, then values\nfrom the provided attributes map are copied to like-named fields in the struct; overwriting\nany values copied from the base_struct.  Raising if the new struct cannot be validated.\n\nThe `from!/2` constructor is useful for event driven systems where it is common to create\na new event from a given \"triggering\" event\n\n* Provided values from `base_struct` will be cast to the appropriate field type.\n* Fields in the newly constructed struct that are not provided map will\n  be set to their default values.\n* Any cast error or missing required field will result in a `raise`","ref":"TypedStructCtor.html#from!/2"},{"type":"function","title":"TypedStructCtor.new/0","doc":"Create a new struct with default values","ref":"TypedStructCtor.html#new/0"},{"type":"function","title":"TypedStructCtor.new/1","doc":"Create a new struct where values from keys in the provided attributes map are copied\nto like-named fields in the struct.\n\n* Provided values will be cast to the appropriate field type.\n* Fields in the newly constructed struct that are not in the provided map will\n  be set to their default values.\n* Required fields that are nil will result in a Ecto.Changeset error","ref":"TypedStructCtor.html#new/1"},{"type":"function","title":"TypedStructCtor.new!/0","doc":"Create a new struct with default values.  Raises if the new struct cannot be validated.","ref":"TypedStructCtor.html#new!/0"},{"type":"function","title":"TypedStructCtor.new!/1","doc":"Create a new struct where values from keys in the provided attributes map are copied\nto like-named fields in the struct.  Raises if the new struct cannot be validated.\n\n* Provided values will be cast to the appropriate field type.\n* Fields in the newly constructed struct that are not in the provided map will\n  be set to their default values.\n* Any cast error or missing required field will result in a raise","ref":"TypedStructCtor.html#new!/1"},{"type":"extras","title":"TypedStructCtor","doc":"# TypedStructCtor\n\nA TypedStruct plugin to add validating constructors to a TypedStruct module\n\nThe [TypedStruct](https://hexdocs.pm/typedstruct/TypedStruct.html) macro wraps field definitions to reduce boilerplate\nneeded to define elixir structs and provides a\n[plugin system](https://hexdocs.pm/typedstruct/TypedStruct.Plugin.html) for clients to extend the DSL.\n\n`TypedStructCtor` uses the `__changeset__` \"reflection\" function added by the plugin\n[TypedStructEctoChangeset](https://hexdocs.pm/typed_struct_ecto_changeset/TypedStructEctoChangeset.html) which enables\nEcto.Changeset.cast on fields defined within the TypedStruct macro.","ref":"readme.html"},{"type":"extras","title":"Try it out in Livebook - TypedStructCtor","doc":"Try the macro out in real time without having to install or write any of your own code\n\nTo get started you need a running instance of [Livebook](https://livebook.dev/)\n\n[![Run in Livebook](https://livebook.dev/badge/v1/blue.svg)](https://livebook.dev/run?url=https://github.com/withbelay/typed_struct_ctor/blob/main/try_it_out.livemd)","ref":"readme.html#try-it-out-in-livebook"},{"type":"extras","title":"Rationale - TypedStructCtor","doc":"[Ecto.Changeset](https://hexdocs.pm/ecto/Ecto.Changeset.html) is a great way to create validated\nstructs.  However, if you've many validated structs they quickly become \"noise\", where writing\nthese functions can be tedious, and bugs are easily introduced as struct changes are easily overlooked.\nWhen the effort needed to write boilerplate tests for boilerplate code is factored in, it can be tempting to skip\nstruct validation altogether.","ref":"readme.html#rationale"},{"type":"extras","title":"Simple Examples - TypedStructCtor","doc":"```elixir\n    defmodule AStruct do\n      use TypedStruct\n\n      typedstruct do\n        plugin(TypedStructEctoChangeset)\n        plugin(TypedStructCtor)\n\n        # A required field with a default value provided by MFA tuple to return a UUID\n        field :id, :string, default_apply: {Ecto.UUID, :generate, []}\n        \n        # A required field with no default, meaning it must be provided to the constructor.\n        # It's an `integer` with known `Ecto.cast` behavior, so for instance, string values are cast \n        # for example string to integer\n        field :integer_field, :integer\n        \n        # An optional field with no default, meaning it will only have a value if provided to the \n        # constructor\n        field :some_string, :string, required: false \n      end\n    end\n\n    iex()> AStruct.new(%{some_string: \"foo\"})\n    {:error,\n     #Ecto.Changeset ,\n       valid?: false\n     >}\n\n    # With `bang` notation and demonstrating Ecto's field cast (string to integer)\n    iex()> AStruct.new!(%{some_string: \"bar\", integer_field: \"42\"})\n    %AStruct{\n      some_string: \"bar\",\n      integer_field: 42,\n      id: \"2e28df41-c024-465e-901d-22c974f1d356\"\n    }\n  ```\n\nThe TypedStruct macro makes it much easier to define structs.  The TypedStructEctoChangeset plugin uses the field\ndefinitions to generate an Ecto.Changeset.cast function for fields in the struct.  And this plugin, TypedStructCtor,\nuses those `cast` functions to generate validating constructors for the enclosing struct created by TypedStruct.\n\nThis plugin adds 5 constructors, `new/0`, `new/1`, `new!/1`, `from/2`, and `from!/2` to the given module.  \nEcto `cast` is called for all attributes provided to the constructors, defaults are applied where needed, and\nvalidation is performed.  \n\nThe `new` functions return {:ok, struct} or {:error, changeset}, while the `new!` functions return the struct, or\nraises if there were issues with `cast` or validation.\n\nThe `new` function takes an optional map of attributes, does\n[Changeset.cast](https://hexdocs.pm/ecto/Ecto.Changeset.html#cast/4) of all values matching the defined\nfields, adds defaults for fields missing values, validates any required fields, and finally does\n[Changeset.apply_action](https://hexdocs.pm/ecto/Ecto.Changeset.html#apply_action/2) to validate the changeset.\nReturns `{:ok,  }` if everything is OK, `{:error,  }` if there were issues with `cast` or validation.\nBecause it's necessary to properly handle mappable fields, if a struct is passed to the `new` function,\n`{:error, :attributes_must_be_a_map}` is returned; use one of the `from` functions for that use case as described below.\n\nThe `from` functions are useful in messaging environments where a new message is created from a some set of\nvalues from a source message.  They are similar to the `new` functions but accept a \"base struct\" as the first argument \nand a map of attributes as the second argument.  The base struct is mapped first to the field values, and the attributes \nare merged on top.","ref":"readme.html#simple-examples"},{"type":"extras","title":"Required Fields - TypedStructCtor","doc":"By default, all fields are required when calling the constructors.  Meaning you'll get a changeset error if the \nfield does not have a default, and you don't provide an attribute value for it in the constructor.\n\nYou can override this by passing `required: false` to the plugin\n\n  ```elixir\n  typedstruct do\n    plugin(TypedStructEctoChangeset)\n    plugin(TypedStructCtor, required: false)\n\n    field :this_field_is_not_required, :string\n  end\n  ```\n\nOr by passing `required: false` to the `field` definition.\n\n  ```elixir\n  typedstruct do\n    plugin(TypedStructEctoChangeset)\n    plugin(TypedStructCtor)\n\n    field :this_field_is_not_required, :string, required: false\n  end\n  ```","ref":"readme.html#required-fields"},{"type":"extras","title":"Field-level Defaults - TypedStructCtor","doc":"`default` and `default_apply` can be provided to the `field` definition to specify a default value for the field.\n\nThough you can specify both `default` and `default_apply` (an MFA tuple), only one will be used.\n`default` will be used with Elixir's struct syntax (e.g. `%AStruct{}`).\n`default_apply` will be invoked when one of the 5 constructor functions is used (e.g. `AStruct.new!()`)\n\nThe `default_apply` function is short-circuited and will only be invoked if the given field was not present in the \nattributes.\n\n  ```elixir\n    defmodule AStructWithDefaulting do\n      use TypedStruct\n\n      typedstruct do\n        plugin(TypedStructEctoChangeset)\n        plugin(TypedStructCtor)\n\n        field :field, :integer, default: 42, default_apply: {SomeModule, :some_function, [\"55\"]}\n      end\n    end\n\n    iex()> %AStructWithDefaulting{}\n    %AStructWithDefaulting{field: 42}\n\n    iex()> AStructWithDefaulting.new!()\n    %AStructWithDefaulting{field: 55}\n  ```","ref":"readme.html#field-level-defaults"},{"type":"extras","title":"Mappable Fields - TypedStructCtor","doc":"As mentioned above, when using `from` and `from!` functions, the first argument is a \"source\" struct whose matching-name\nfields will be copied first into the struct being constructed.  By default, all matching-name fields are copied, but\nthe `mappable?` boolean attribute can be used to specify which fields are not copied.  This is useful when you want the\nnewly constructed struct to have different values for a field than the source struct such as `created_at` or `id`.\n\nNot mapping over the source struct values will mean the newly constructed struct will leave the new fields empty\nunless defaulted or provided as attributes to the constructor.\n\n  ```elixir\n    defmodule AStructWithMappableFields do\n      use TypedStruct\n    \n      typedstruct do\n        plugin(TypedStructEctoChangeset)\n        plugin(TypedStructCtor)\n    \n        field(:id, :string, mappable?: false, default_apply: {Ecto.UUID, :generate, []})\n        field(:created_at, :utc_datetime_usec, mappable?: false, default_apply: {DateTime, :utc_now, []})\n        field(:reason, :string)\n      end\n    end\n\n    # In the example below, the `id` and `created_at` fields are `mappable?: false` so they are not \n    # copied from the source struct.  So in the new struct, `:reason` is copied from the source \n    # struct, `:id` is provided in the attributes map, and `:created_at`, being nil after all the\n    # copying is done, causes its default to be used instead, resulting in a new date.\n    iex()> source_struct = AStructWithMappableFields.new!(%{reason: \"because\"})\n    %AStructWithMappableFields{\n      reason: \"because\",\n      created_at: ~U[2023-11-18 04:57:16.754681Z],\n      id: \"ffe94776-5d6e-4d84-9aeb-2862d874577f\"\n    }\n    \n    iex()>  Process.sleep(5)\n    iex()>  mapped = AStructWithMappableFields.from!(source_struct, %{id: \"id from attributes\"})\n    %AStructWithMappableFields{\n      reason: \"because\",\n      created_at: ~U[2023-11-18 04:57:16.766353Z],\n      id: \"id from attributes\"\n    }\n\n    iex()> Process.sleep(5)\n    iex()> mapped = AStructWithMappableFields.from!(source_struct, %{reason: \"I said so\"})\n    %AStructWithMappableFields{\n      reason: \"I said so\",\n      created_at: ~U[2023-11-18 04:57:16.772312Z],\n      id: \"a09be86b-373a-48f0-9d74-faee10037421\"\n    }\n  ```","ref":"readme.html#mappable-fields"},{"type":"extras","title":"Installation - TypedStructCtor","doc":"Because this plugin supports the interface defined by the `TypedStruct` macro, installation assumes you've already\nadded that dependency.\n\nWhile you can use the original [typed_struct](https://hex.pm/packages/typed_struct) library, it seems to no longer be\nmaintained.  However, there is a fork [here](https://hex.pm/packages/typedstruct) that is quite active.\n\nThe package can be installed by adding `typed_struct_ctor` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    # Choose either of the following `TypedStruct` libraries \n    # both use the same name for the macro - `typedstruct` but\n    # but are mutually exclusive:\n    \n    # The original, but no longer maintained library\n    {:typed_struct, \"~> 0.3.0\"},\n      \n    # Or the newer forked library\n    {:typedstruct, \"~> 0.5.2\"},\n\n    # And add this library  \n    {:typed_struct_ctor, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"License","doc":"# MIT License\n\nCopyright © 2018-2022 Lee Eggebroten and Contributors\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html"}],"content_type":"text/markdown"}